import os
import sys
import subprocess

# returns in_path,gpg_path
def find_gpg():
	if sys.platform == 'win32':
		in_path,gpgpath = find_gpg_windows()
		# Removed due to gnupg.py v0.37
		#if gpgpath != None and gpgpath.find(' ') >= 0:
		#	gpgpath = '"' + gpgpath + '"'
	elif sys.platform == 'darwin':
		in_path,gpgpath = find_gpg_macos()
	else:
		in_path,gpgpath = find_gpg_unix()
	return in_path,gpgpath

def find_gpg_windows():
	try:
		for line in os.environ['PATH'].split(os.pathsep):
			gpgpath = line + os.sep + 'gpg.exe'
			if os.path.isfile(gpgpath):
				return True,gpgpath
	except Exception:
		pass

	try:
		# Look for local copy in the install directory first
		program_path = os.path.dirname(os.path.realpath(sys.argv[0]))
		program_path_bin = program_path + os.sep + 'bin'

		#DBGOFF#for line in [ program_path,'C:\\Program Files (x86)\\GNU\\GnuPG.v2\\bin','C:\\Program Files\\GNU\\GnuPG',program_path_bin,'C:\\Program Files (x86)\\GNU\\GnuPG\\bin','C:\\Program Files\\GNU\\GnuPG\\bin' ]:
		for line in [ program_path,'C:\\Program Files (x86)\\GNU\\GnuPG','C:\\Program Files\\GNU\\GnuPG','C:\\Program Files (x86)\\GnuPG\\bin','C:\\Program Files\\GnuPG\\bin',program_path_bin,'C:\\Program Files (x86)\\GNU\\GnuPG\\bin','C:\\Program Files\\GNU\\GnuPG\\bin' ]:
			gpgpath = line + os.sep + 'gpg.exe'
			if os.path.isfile(gpgpath):
				return False,gpgpath
	except Exception:
		pass

	return False,None

def find_gpg_macos():
	try:
		program_path = os.path.dirname(os.path.realpath(sys.argv[0]))
		bin_path = program_path + os.sep + 'bin'
		search_paths = [ program_path,bin_path,'_NOT_IN_PATH_' ]
		search_paths.append(os.environ['PATH'].split(os.pathsep))
		inpath = True
		for line in search_paths:
			if line == '_NOT_IN_PATH_':
				inpath = False
			gpgpath = line + os.sep + 'gpg'
			if os.path.isfile(gpgpath):
				return inpath,gpgpath
	except Exception:
		pass
	return False,None

def find_gpg_unix():
	try:
		for line in os.environ['PATH'].split(os.pathsep):
			gpgpath = line + os.sep + 'gpg'
			if os.path.isfile(gpgpath):
				return True,gpgpath
	except Exception:
		pass
	return False,None

# This is the homedir for Confidant Mail files. It is unrelated to GPG.
def find_default_homedir():
	appdata = None
	if sys.platform == 'win32':
		varname = 'APPDATA'
	else:
		varname = 'HOME'
	if varname in os.environ:
		basedir = os.environ[varname]
	if basedir != None and os.path.exists(basedir):
		homedir = basedir + os.sep + 'confidantmail'
		if os.path.exists(homedir):
			return homedir
		else:
			try:
				os.mkdir(homedir)
			except Exception:
				pass
			if os.path.exists(homedir):
				return homedir
			else:
				return None
	else:
		return None
			
def find_gpg_version(gpgpath):
	isv2 = False
	gpgver = ""
	libgver = ""
	try:
		cmdline = [ gpgpath,"--version" ]
		if sys.platform == 'win32':
			# http://stackoverflow.com/questions/7006238/how-do-i-hide-the-console-when-i-use-os-system-or-subprocess-call/7006424#7006424
			CREATE_NO_WINDOW = 0x08000000
			cmdh = subprocess.Popen(cmdline,bufsize=16384,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,shell = False,creationflags = CREATE_NO_WINDOW)
		else:
			cmdh = subprocess.Popen(cmdline,bufsize=16384,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,shell = False)
		cmdh.stdin.close() # undefined stdin -> Windows fails
		gpgver_o = cmdh.stdout.read()
		cmdh.stdout.close()
		for line in gpgver_o.split('\n'):
			line = line.rstrip("\r\n")
			if line[0:15] == 'gpg (GnuPG) 2.1':
				isv2 = True
				gpgver = line[12:]
			elif line[0:15] == 'gpg (GnuPG) 1.4':
				isv2 = False
				gpgver = line[12:]
			elif line[0:10] == 'libgcrypt ':
				libgver = line[10:]
	except Exception:
		pass
	return isv2,gpgver,libgver

# Debian uses version 1.4.18-7+deb8u2 which is patched for CVE-2016-6313
# This returns True if it detects such a patched version, False otherwise
def check_gpg_version_special_case():
	dpkg_query = '/usr/bin/dpkg-query';
	cmdline = [ dpkg_query,'-s','gnupg' ];
	if sys.platform[0:5] != 'linux':
		return False
	if os.path.exists(dpkg_query) == False:
		return False
	cmdh = subprocess.Popen(cmdline,bufsize=16384,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,shell = False)
	cmdh.stdin.close() # undefined stdin -> Windows fails
	gpgver_o = cmdh.stdout.read()
	cmdh.stdout.close()
	for line in gpgver_o.split('\n'):
		line = line.rstrip("\r\n")
		if line[0:9] == 'Version: ':
			if line[9:] >= '1.4.18-7+deb8u2':
				return True
	return False

# EOF
